#!/bin/bash
###########################################################
# ae-fstab: A tool to set the file system encoding to utf8
# 
# A silly script by Moayyad Al-Sadi <ErrorMsg@gawab.com>
# written for resala core linux but should work with 
# any linux.
# 
# Usually most end users want everybody to access
# windowz partitions and mount/unmount floppys and cdroms;
# this script will do that.
#
# (c) Copyright under terms of GPL visit <www.gnu.org/copyleft>
#
###########################################################
MYNAME=`basename "$0"`
ONLY_U=0
FSTAB="/etc/fstab"
NONCOMMENT=0

function usage() {
    echo "$MYNAME: A tool to set the file system encoding to utf8
usage:
    $MYNAME -h			This help message
    $MYNAME [-u] [FSTAB]	Enable utf8 file system
options:
    -u				Just do the utf8 and skip umask stuff
    FSTAB			The file to use instead of /etc/fstab
example:
    just type $MYNAME as root will do the job
"

}
function myfstab() {
    cat "$FSTAB" | tr -s " \t" " " | sed -e "s/^ *//g"
}
function getfstab() {
    echo -n "$MYNAME: Parsing [$FSTAB]  "
    LINEN=$( wc -l "$FSTAB" | sed -e 's/[^0-9]//g' )
    j=0
    for ((i=1; i<=LINEN; i++)) ; do
	echo -n "."
	FSLINE=$( myfstab | cut -d "
" -f "$i" )
	echo "$FSLINE" | grep -q "^#.*" || {
	FS_LINE[$j]="$i"
        FS_DEV[$j]=$( echo "$FSLINE" | cut -d ' ' -f '1' )
        FS_MNT[$j]=$( echo "$FSLINE" | cut -d ' ' -f '2' )
	FS_TYP[$j]=$( echo "$FSLINE" | cut -d ' ' -f '3' )
	FS_OPT[$j]=$( echo "$FSLINE" | cut -d ' ' -f '4' )
	FS_DMB[$j]=$( echo "$FSLINE" | cut -d ' ' -f '5' )
	FS_CHK[$j]=$( echo "$FSLINE" | cut -d ' ' -f '6' )
	#echo "$j [${FS_LINE[$j]}] \
#[${FS_DEV[$j]}] [${FS_MNT[$j]}] [${FS_TYP[$j]}] \
#[${FS_OPT[$j]}] [${FS_DMB[$j]}] [${FS_CHK[$j]}]"
	let j=j+1
	}
	echo -n "."
    done
    NONCOMMENT=$j
    echo " OK"
}
function rmopt() {
    R=`echo "$1" | sed -e "s/$2\(=[^,]*\)\?,//g" -e "s/\(,\)\?$2\(=[^,]*\)\?$//g"`
    [ "$R" == "" ] && R="defaults"
    echo "$R"
}
function addopt() {
    echo "$1,$2"
}
function addutf() {
    echo -n "$MYNAME: Unicoding [$FSTAB]  "
    for ((i=0; i<NONCOMMENT; i++)) ; do
	echo -n "."
	Here=0
	echo "${FS_OPT[$i]}" | grep -q "noauto" && Here=1
	[[ "${FS_TYP[$i]}" == "auto" || "${FS_TYP[$i]}" == "msdos" ]] && Here=1
	[[ "${FS_TYP[$i]}" == "ntfs" || "${FS_TYP[$i]}" == "iso9660" ]] && Here=1
	[[ "${FS_TYP[$i]}" == "vfat" ]] && Here=2

	[ "$Here" -eq 2 ] && {
	    FS_OPT[$i]=`rmopt ${FS_OPT[$i]} iocharset`
	    FS_OPT[$i]=`rmopt ${FS_OPT[$i]} codepage`
	    FS_OPT[$i]=`rmopt ${FS_OPT[$i]} utf8`
    	    FS_OPT[$i]=`addopt ${FS_OPT[$i]} utf8`
	    FS_OPT[$i]=`addopt ${FS_OPT[$i]} codepage=864`
	    umount ${FS_MNT[$i]} > /dev/null 2>&1
	}
	[ "$Here" -eq 1 ] && {
	    FS_OPT[$i]=`rmopt ${FS_OPT[$i]} iocharset`
	    FS_OPT[$i]=`rmopt ${FS_OPT[$i]} codepage`
	    FS_OPT[$i]=`rmopt ${FS_OPT[$i]} utf8`
    	    FS_OPT[$i]=`addopt ${FS_OPT[$i]} codepage=864`
    	    FS_OPT[$i]=`addopt ${FS_OPT[$i]} iocharset=utf8`
	    umount ${FS_MNT[$i]} > /dev/null 2>&1
	} 
	echo -n "."
    done
    echo " OK"
}
function extra() {
    echo -n "$MYNAME: Extra fixing [$FSTAB]  "
    for ((i=0; i<NONCOMMENT; i++)) ; do
        echo -n "."
	Here=0
	echo "${FS_OPT[$i]}" | grep -q "noauto" && Here=1
	[[ "${FS_TYP[$i]}" == "auto" || "${FS_TYP[$i]}" == "msdos" ]] && Here=1
	[[ "${FS_TYP[$i]}" == "ntfs" || "${FS_TYP[$i]}" == "iso9660" ]] && Here=1
	[[ "${FS_TYP[$i]}" == "vfat" ]] && Here=1

	[ "$Here" -eq 1 ] && {
	    FS_OPT[$i]=`rmopt ${FS_OPT[$i]} umask`
	    FS_OPT[$i]=`rmopt ${FS_OPT[$i]} uid`
	    FS_OPT[$i]=`rmopt ${FS_OPT[$i]} gid`
	    FS_OPT[$i]=`rmopt ${FS_OPT[$i]} exec`
	    FS_OPT[$i]=`rmopt ${FS_OPT[$i]} noexec`
	    FS_OPT[$i]=`rmopt ${FS_OPT[$i]} user`
	    FS_OPT[$i]=`rmopt ${FS_OPT[$i]} users`
    	    FS_OPT[$i]=`addopt ${FS_OPT[$i]} umask=0`
	    FS_OPT[$i]=`addopt ${FS_OPT[$i]} users`
	    umount ${FS_MNT[$i]} > /dev/null 2>&1
	}
        echo -n "."	   
    done
    echo " OK"
}
function writetab() {
    FS_CONT=`cat $FSTAB`
    for ((i=0; i<NONCOMMENT; i++)) ; do
	SedE="${FS_LINE[$i]}s|^.*$|${FS_DEV[$i]}	${FS_MNT[$i]}	${FS_TYP[$i]}\
    ${FS_OPT[$i]}		${FS_DMB[$i]} ${FS_CHK[$i]}|g"
	FS_CONT=$(echo -n "$FS_CONT" | sed -e "$SedE")
    done
    echo "$FS_CONT" > "$FSTAB"
}
[ "$1" == "-h" ] && usage && exit 0
[ "$1" == "-u" ] && ONLY_U=1 && shift
[ "$1" != "" ] && FSTAB="$1" && shift
[ "$1" == "-u" ] && ONLY_U=1 && shift
[ $#  -ne 0 ] && echo "$MYNAME: Bad arguments." 1>&2 && usage && exit 1
[ ! -f "$FSTAB" ] && echo "$MYNAME:  File [$FSTAB] not found." 1>&2 && exit 1
[ ! -w "$FSTAB" ] &&
    echo "$MYNAME: Could not write to '$FSTAB' - Permission denieded.
$MYNAME: Try to be a root, type 'su' 1st." 1>&2 && exit 1
getfstab
addutf
[ "$ONLY_U" -ne 1 ] && extra
writetab
mount -a
echo "done. You *may* need to remount or reboot"
