#!/bin/bash
#####################################################################
# fatfinder: A tool automaticlly find windows partitions
#
# (c) Copyright, Moayyad Saleh Al-Sadi <alsadi@gmail.com>
# under terms of GPL visit <www.gnu.org/copyleft>
# 
# A silly script written for resala core Linux but should work with 
# any Linux.
# 
# This script uses Linux proc file system and GNU parted
# to get information, then updated your fstab file.
#
# Usually most end users want everybody to access
# windowz partitions and mount/unmount floppys and cdroms;
# this script will do that also.
#
# This script does NOT create fstab from scratch, it parse
# a pre-existing fstab file and it will keep your comments.
# WARNING: THIS TOOL IS NOT COMPITABLE WITH DEVFS
# it will use classical '/dev/hd*' block files
# if there is a 'devfs' device in your fstab this script
# won't see it and you will get 'foo is already mounted'
# simply you may comment any entry that are related
# to windows partitions and start it with a minimal fstab.
#
# (c) Copyright under terms of GPL visit <www.gnu.org/copyleft>
#
####################################################################
MYNAME=`basename "$0"`
QUIET=0
FSTAB="/etc/fstab"
TEST=0
NONCOMMENT=0

function usage() {
    echo "$MYNAME: A tool automaticlly find windows partitions
usage:
    $MYNAME -h			This help message
    $MYNAME [-p] [-q] [FSTAB]	update fstab and create mount point
options:
    -p				do nothing just print.
    -q				be quiet.
    FSTAB			The file to use instead of /etc/fstab (for testing)
    
example:
    just type $MYNAME as root will do the job
"

}

function probe_all() {
    [ $QUIET -eq 0 ] && echo -n "$MYNAME: finding IDE Hard drives : "
    HDD=""
    cd /proc/ide || {
	echo -n "$MYNAME: proc file system is not mounted." 1>&2
	exit 1;
    }
    for i in hd*
    do
        if [ -f "$i/media" ]; then
    	    dv=`cat "$i/media"`
    	    [ "disk" == "$dv" ] && HDD="$HDD $i"
	fi
    done
    [ $QUIET -eq 0 ] && echo " OK. [$HDD ]"
    cd -
    ALPHA="c d e f g i j l m n o p q r s t u v w x y z"
    DEV_N=1
    for i in $HDD
    do
        if [ -b "/dev/$i" ]; then
    	    PARTOUT=`/sbin/parted -s "/dev/$i" print | sed -e '1,3d' | tr -s "\010" " " | cut -d " " -f 1,5 | grep -i -e '[0-9] \(fat\|ntfs\)'` || continue
	    # echo "$PARTOUT"
	    LINES=$( echo "$PARTOUT" | wc -l | sed -e 's/[^0-9]//g' )
	    for ((j=1; j<=LINES; j++)) ; do
		WIN_DEV[$DEV_N]=`echo -n "$PARTOUT" | cut -d "
" -f $j | cut -d ' ' -f 1`
		WIN_DEV[$DEV_N]="/dev/${i}${WIN_DEV[$DEV_N]}"
		echo "$PARTOUT" | cut -d ' ' -f 2 | grep -q -i fat && FS_TYPE='vfat' || FS_TYPE='ntfs'
		DEV_MNT[$DEV_N]=`echo -n "$ALPHA" | cut -d ' ' -f $DEV_N`
		DEV_MNT[$DEV_N]="/mnt/win_${DEV_MNT[$DEV_N]}"
		DEV_FS[$DEV_N]="$FS_TYPE"
		[ $QUIET -eq 0 ] && 
		# echo "[$DEV_N][${WIN_DEV[$DEV_N]}][${DEV_MNT[$DEV_N]}][${DEV_FS[$DEV_N]}]"
		let DEV_N=DEV_N+1
	    done
	else
	    echo "$MYNAME: The block device '/dev/$i' not found. use mknod." 1>&2
	    exit 1
	fi
    done
}
function getfstab() {
    [ $QUIET -eq 0 ] && echo -n "$MYNAME: Parsing [$FSTAB]  "
    myfstab=`cat "$FSTAB" | tr -s " \t" " " | sed -e "s/^ *//g"`
    LINES=$( wc -l "$FSTAB" | sed -e 's/[^0-9]//g' )
    j=0
    for ((i=1; i<=LINES; i++)) ; do
	echo -n "."
	FSLINE=$( echo "$myfstab" | cut -d "
" -f "$i" )
	echo "$FSLINE" | grep -q "^#.*" || {
	FS_LINE[$j]="$i"
        FS_DEV[$j]=$( echo "$FSLINE" | cut -d ' ' -f '1' )
        FS_MNT[$j]=$( echo "$FSLINE" | cut -d ' ' -f '2' )
	FS_TYP[$j]=$( echo "$FSLINE" | cut -d ' ' -f '3' )
	FS_OPT[$j]=$( echo "$FSLINE" | cut -d ' ' -f '4' )
	FS_DMB[$j]=$( echo "$FSLINE" | cut -d ' ' -f '5' )
	FS_CHK[$j]=$( echo "$FSLINE" | cut -d ' ' -f '6' )
	let j=j+1
	}
	[ $QUIET -eq 0 ] && echo -n "."
    done
    NONCOMMENT=$j
    [ $QUIET -eq 0 ] && echo " OK"
}
function is_in_fstab() {
    for ((j=0; j<NONCOMMENT; j++)) ; do
        [ "${FS_DEV[$j]}" == "$1" ] && return 0;
    done
    return 1;
}
function addwindev() {
    CHANGE=0
    [ $QUIET -eq 0 ] && echo "$MYNAME: adding devices to fstab : "
    for ((i=1; i<DEV_N; i++)) ; do
	[ $QUIET -eq 0 ] && echo -n "$MYNAME: checking for [${WIN_DEV[$i]}] entry :"
	is_in_fstab "${WIN_DEV[$i]}" && {
	    [ $QUIET -eq 0 ] && echo "found (SKIPED)"
	} || {
	    CHANGE=1
	    [ $QUIET -eq 0 ] && echo "NOT found"
	    [ $QUIET -eq 0 ] && echo "$MYNAME: creating [${DEV_MNT[$i]}] directory."
	    [ $TEST -eq 0 ] && mkdir "${DEV_MNT[$i]}" >/dev/null
	    [ $QUIET -eq 0 ] && echo "$MYNAME: new entry [${WIN_DEV[$i]} ${DEV_MNT[$i]} ${DEV_FS[$i]} noexec,umask=0 0 0]"
	    [ $TEST -eq 0 ] && echo "# the next line added by fatfinder send comments to <alsadi@gmail.com>" >> "$FSTAB"
	    [ "${DEV_FS[$i]}" == "vfat" ] && FS_OPT="rw" || FS_OPT="ro"
    	    [ $TEST -eq 0 ] && echo "${WIN_DEV[$i]} ${DEV_MNT[$i]} ${DEV_FS[$i]} noexec,umask=0,$FS_OPT 0 0" >> "$FSTAB"
	    umount "${WIN_DEV[$i]}" > /dev/null
	    mount -t "${DEV_FS[$i]}" -o "$FS_OPT" "${WIN_DEV[$i]}" "${DEV_MNT[$i]}" || echo "$MYNAME: can't mount, it's may be because your RH/FC kernel won't support ntfs"
	}
    done
}

    [ "$1" == "-h" ] && usage && exit 0
    [ "$1" == "-q" ] && QUIET=1 && shift
    [ "$1" == "-p" ] && TEST=1 && shift
    [ "$1" == "-q" ] && QUIET=1 && shift
    [ "$1" != "" ] && FSTAB="$1" && shift
    [ "$1" == "-q" ] && QUIET=1 && shift
    [ "$1" == "-p" ] && TEST=1 && shift
    [ "$1" == "-q" ] && QUIET=1 && shift
    [ $#  -ne 0 ] && echo "$MYNAME: Bad arguments." 1>&2 && usage && exit 1
    [ ! -f "$FSTAB" ] && echo "$MYNAME:  File [$FSTAB] not found." 1>&2 && exit 1
    [ ! -w "$FSTAB" ] &&
        echo "$MYNAME: Could not write to '$FSTAB' - Permission denieded.
    $MYNAME: Try to be a root, type 'su' 1st." 1>&2 && exit 1
    [ "$UID" -ne 0 ] && echo "$MYNAME: be a root, type 'su' 1st."  1>&2 && exit 1

    probe_all
    getfstab
    addwindev
    [ $CHANGE -eq 0 ] && {
	[ $QUIET -eq 0 ] && echo "$MYNAME: I did nothing at all. if you think this is not correct try commenting some lines in fstab"
	exit 1
    } || {
    mount -a
    echo "done. You *may* need to remount or reboot"
    exit 0    
    }
